@@ -309,8 +309,8 @@ ThreadId VG_(alloc_ThreadState) ( void )
    Int i;
    for (i = 1; i < VG_N_THREADS; i++) {
       if (VG_(threads)[i].status == VgTs_Empty) {
-	 VG_(threads)[i].status = VgTs_Init;
-	 VG_(threads)[i].exitreason = VgSrc_None;
+         VG_(threads)[i].status = VgTs_Init;
+         VG_(threads)[i].exitreason = VgSrc_None;
          if (VG_(threads)[i].thread_name)
             VG_(free)(VG_(threads)[i].thread_name);
          VG_(threads)[i].thread_name = NULL;
@@ -469,9 +469,9 @@ void VG_(get_thread_out_of_syscall)(ThreadId tid)
 
    if (VG_(threads)[tid].status == VgTs_WaitSys) {
       if (VG_(clo_trace_signals)) {
-	 VG_(message)(Vg_DebugMsg, 
+         VG_(message)(Vg_DebugMsg,
                       "get_thread_out_of_syscall zaps tid %u lwp %d\n",
-		      tid, VG_(threads)[tid].os_state.lwpid);
+                      tid, VG_(threads)[tid].os_state.lwpid);
       }
 #     if defined(VGO_darwin)
       {
@@ -519,7 +519,7 @@ void VG_(vg_yield)(void)
    /* 
       Tell the kernel we're yielding.
     */
-#  if defined(VGO_linux) || defined(VGO_darwin)
+#  if defined(VGO_linux) || defined(VGO_darwin) || defined(VGO_freebsd)
    VG_(do_syscall0)(__NR_sched_yield);
 #  elif defined(VGO_solaris)
    VG_(do_syscall0)(__NR_yield);
@@ -545,6 +545,7 @@ static void block_signals(void)
    VG_(sigdelset)(&mask, VKI_SIGFPE);
    VG_(sigdelset)(&mask, VKI_SIGILL);
    VG_(sigdelset)(&mask, VKI_SIGTRAP);
+   VG_(sigdelset)(&mask, VKI_SIGSYS);
 
    /* Can't block these anyway */
    VG_(sigdelset)(&mask, VKI_SIGSTOP);
@@ -560,6 +561,8 @@ static void os_state_clear(ThreadState *tst)
    tst->os_state.stk_id = NULL_STK_ID;
 #  if defined(VGO_linux)
    /* no other fields to clear */
+#  elif defined(VGO_freebsd)
+   /* no other fields to clear */
 #  elif defined(VGO_darwin)
    tst->os_state.post_mach_trap_fn = NULL;
    tst->os_state.pthread           = 0;
@@ -654,7 +657,7 @@ static void sched_fork_cleanup(ThreadId me)
    for (tid = 1; tid < VG_N_THREADS; tid++) {
       if (tid != me) {
          mostly_clear_thread_record(tid);
-	 VG_(threads)[tid].status = VgTs_Empty;
+         VG_(threads)[tid].status = VgTs_Empty;
          VG_(clear_syscallInfo)(tid);
       }
    }
@@ -2050,6 +2053,7 @@ void do_client_request ( ThreadId tid )
       case VG_USERREQ__STACK_REGISTER: {
          UWord sid = VG_(register_stack)((Addr)arg[1], (Addr)arg[2]);
          SET_CLREQ_RETVAL( tid, sid );
+         VG_TRACK(register_stack, (Addr)arg[1], (Addr)arg[2]);
          break; }
 
       case VG_USERREQ__STACK_DEREGISTER: {
@@ -2070,11 +2074,15 @@ void do_client_request ( ThreadId tid )
 	 info->tl_realloc              = VG_(tdict).tool_realloc;
 	 info->tl_memalign             = VG_(tdict).tool_memalign;
 	 info->tl___builtin_new        = VG_(tdict).tool___builtin_new;
+	 info->tl___builtin_new_aligned = VG_(tdict).tool___builtin_new_aligned;
 	 info->tl___builtin_vec_new    = VG_(tdict).tool___builtin_vec_new;
+	 info->tl___builtin_vec_new_aligned    = VG_(tdict).tool___builtin_vec_new_aligned;
 	 info->tl_free                 = VG_(tdict).tool_free;
 	 info->tl___builtin_delete     = VG_(tdict).tool___builtin_delete;
+	 info->tl___builtin_delete_aligned     = VG_(tdict).tool___builtin_delete_aligned;
 	 info->tl___builtin_vec_delete = VG_(tdict).tool___builtin_vec_delete;
-         info->tl_malloc_usable_size   = VG_(tdict).tool_malloc_usable_size;
+	 info->tl___builtin_vec_delete_aligned = VG_(tdict).tool___builtin_vec_delete_aligned;
+	 info->tl_malloc_usable_size   = VG_(tdict).tool_malloc_usable_size;
 
 	 info->mallinfo                = VG_(mallinfo);
 	 info->clo_trace_malloc        = VG_(clo_trace_malloc);
