@@ -7,7 +7,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2000-2015 Julian Seward
+   Copyright (C) 2000-2017 Julian Seward
       jseward@acm.org
 
    This program is free software; you can redistribute it and/or
@@ -21,9 +21,7 @@
    General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-   02111-1307, USA.
+   along with this program; if not, see <http://www.gnu.org/licenses/>.
 
    The GNU General Public License is contained in the file COPYING.
 */
@@ -31,24 +29,65 @@
 #ifndef __PUB_TOOL_DEBUGINFO_H
 #define __PUB_TOOL_DEBUGINFO_H
 
-#include "pub_tool_basics.h"   // VG_ macro
+#include "pub_tool_basics.h"   // VG_ macro, DiEpoch
 #include "pub_tool_xarray.h"   // XArray
 
+
 /*====================================================================*/
-/*=== Obtaining debug information                                  ===*/
+/*=== Debuginfo epochs.                                            ===*/
 /*====================================================================*/
 
+// This returns the current epoch.
+DiEpoch VG_(current_DiEpoch)(void);
+
+
+/*====================================================================*/
+/*=== Obtaining information pertaining to source artefacts.        ===*/
+/*====================================================================*/
+
+/* IMPORTANT COMMENT about memory persistence and ownership.
+
+   Many functions below are returning a string in a HChar** argument.
+   This memory must not be freed by the caller : it belongs to the debuginfo
+   module. The returned string is *not* guaranteed to be persistent.
+   The exact persistence depends on the kind of information returned,
+   and of the internal implementation of the debuginfo module.
+   In other words: use the memory directly after the call, and if in doubt,
+   save it away.
+
+   In general, all returned strings will be invalidated when the
+   DebugInfo they correspond to is discarded. This is the case for
+   the filename, dirname, fnname and objname.
+   An objname might also be invalidated by changes to the address
+   space manager segments, e.g. if a segment is merged with another
+   segment.
+
+   Retrieving a fnname might imply a call to the c++ demangler.
+   A returned fnname is invalidated if any other call to the demangler
+   is done. In particular, this means that the memory returned by one of
+   the VG_(get_fnname...) functions is invalidated by :
+     * another call to any of the functions VG_(get_fnname...).
+     * any other call that will directly or indirectly invoke the
+       c++ demangler. Such an indirect call to the demangler can a.o. be
+       done by calls to pub_tool_errormgr.h functions.
+   So, among others, the following is WRONG:
+       VG_(get_fnname)(a1, &fnname1);
+       VG_(get_fnname)(a2, &fnname2);
+       ... it is WRONG to use fnname1 here ....
+*/
+
 /* Get the file/function/line number of the instruction at address
    'a'.  For these four, if debug info for the address is found, it
    copies the info into the buffer/UInt and returns True.  If not, it
    returns False.  VG_(get_fnname) always
    demangles C++ function names.  VG_(get_fnname_w_offset) is the
-   same, except it appends "+N" to symbol names to indicate offsets.  */
-extern Bool VG_(get_filename) ( Addr a, const HChar** filename );
-extern Bool VG_(get_fnname)   ( Addr a, const HChar** fnname );
-extern Bool VG_(get_linenum)  ( Addr a, UInt* linenum );
+   same, except it appends "+N" to symbol names to indicate offsets.
+   NOTE: See IMPORTANT COMMENT above about persistence and ownership. */
+extern Bool VG_(get_filename) ( DiEpoch ep, Addr a, const HChar** filename );
+extern Bool VG_(get_fnname)   ( DiEpoch ep, Addr a, const HChar** fnname );
+extern Bool VG_(get_linenum)  ( DiEpoch ep, Addr a, UInt* linenum );
 extern Bool VG_(get_fnname_w_offset)
-                              ( Addr a, const HChar** fnname );
+                              ( DiEpoch ep, Addr a, const HChar** fnname );
 
 /* This one is the most general.  It gives filename, line number and
    optionally directory name.  filename and linenum may not be NULL.
@@ -58,14 +97,12 @@ extern Bool VG_(get_fnname_w_offset)
    it is available; if not available, '\0' is written to the first
    byte.
 
-   The character strings returned in *filename and *dirname are not
-   persistent. They will be freed when the DebugInfo they belong to
-   is discarded.
+   NOTE: See IMPORTANT COMMENT above about persistence and ownership.
 
    Returned value indicates whether any filename/line info could be
    found. */
 extern Bool VG_(get_filename_linenum)
-                              ( Addr a, 
+                              ( DiEpoch ep, Addr a,
                                 /*OUT*/const HChar** filename,
                                 /*OUT*/const HChar** dirname,
                                 /*OUT*/UInt* linenum );
@@ -76,8 +113,10 @@ extern Bool VG_(get_filename_linenum)
    instruction in a function.  Use this to instrument the start of
    a particular function.  Nb: if an executable/shared object is stripped
    of its symbols, this function will not be able to recognise function
-   entry points within it. */
-extern Bool VG_(get_fnname_if_entry) ( Addr a, const HChar** fnname );
+   entry points within it.
+   NOTE: See IMPORTANT COMMENT above about persistence and ownership. */
+extern Bool VG_(get_fnname_if_entry) ( DiEpoch ep, Addr a,
+                                       const HChar** fnname );
 
 typedef
    enum {
@@ -90,13 +129,13 @@ typedef
 extern Vg_FnNameKind VG_(get_fnname_kind) ( const HChar* name );
 
 /* Like VG_(get_fnname_kind), but takes a code address. */
-extern Vg_FnNameKind VG_(get_fnname_kind_from_IP) ( Addr ip );
+extern Vg_FnNameKind VG_(get_fnname_kind_from_IP) ( DiEpoch ep, Addr ip );
 
 /* Looks up data_addr in the collection of data symbols, and if found
    puts its name (or as much as will fit) into dname[0 .. n_dname-1],
    which is guaranteed to be zero terminated.  Also data_addr's offset
    from the symbol start is put into *offset. */
-extern Bool VG_(get_datasym_and_offset)( Addr data_addr,
+extern Bool VG_(get_datasym_and_offset)( DiEpoch ep, Addr data_addr,
                                          /*OUT*/const HChar** dname,
                                          /*OUT*/PtrdiffT* offset );
 
@@ -116,12 +155,17 @@ extern Bool VG_(get_datasym_and_offset)( Addr data_addr,
 Bool VG_(get_data_description)( 
         /*MOD*/ XArray* /* of HChar */ dname1v,
         /*MOD*/ XArray* /* of HChar */ dname2v,
-        Addr data_addr
+        DiEpoch ep, Addr data_addr
      );
 
+/* True if we have some Call Frame unwindo debuginfo for Addr a */
+extern Bool VG_(has_CF_info)(Addr a);
+
 /* Succeeds if the address is within a shared object or the main executable.
-   It doesn't matter if debug info is present or not. */
-extern Bool VG_(get_objname)  ( Addr a, const HChar** objname );
+   It first searches if Addr a belongs to the text segment of debug info.
+   If not found, it asks the address space manager whether it
+   knows the name of the file associated with this mapping. */
+extern Bool VG_(get_objname) ( DiEpoch ep, Addr a, const HChar** objname );
 
 
 /* Cursor allowing to describe inlined function calls at an IP,
@@ -136,7 +180,7 @@ typedef  struct _InlIPCursor InlIPCursor;
    eip can possibly corresponds to inlined function call(s).
    To describe eip and the inlined function calls, the following must
    be done:
-       InlIPCursor *iipc = VG_(new_IIPC)(eip);
+       InlIPCursor *iipc = VG_(new_IIPC)(ep, eip);
        do {
           buf = VG_(describe_IP)(eip, iipc);
           ... use buf ...
@@ -149,12 +193,16 @@ typedef  struct _InlIPCursor InlIPCursor;
    Note, that the returned string is allocated in a static buffer local to
    VG_(describe_IP). That buffer will be overwritten with every invocation.
    Therefore, callers need to possibly stash away the string.
+
+   Since this maps a code location to a source artefact (function names),
+   new_IIPC requires a DiEpoch argument (ep) too.
 */
-extern const HChar* VG_(describe_IP)(Addr eip, const InlIPCursor* iipc);
+extern const HChar* VG_(describe_IP)(DiEpoch ep, Addr eip,
+                                     const InlIPCursor* iipc);
 
 /* Builds a IIPC (Inlined IP Cursor) to describe eip and all the inlined calls
    at eip. Such a cursor must be deleted after use using VG_(delete_IIPC). */
-extern InlIPCursor* VG_(new_IIPC)(Addr eip);
+extern InlIPCursor* VG_(new_IIPC)(DiEpoch ep, Addr eip);
 /* Move the cursor to the next call to describe.
    Returns True if there are still calls to describe.
    False if nothing to describe anymore. */
@@ -206,7 +254,7 @@ VG_(di_get_global_blocks_from_dihandle) ( ULong di_handle,
 
 
 /*====================================================================*/
-/*=== Obtaining debug information                                  ===*/
+/*=== Obtaining information pertaining to shared objects.          ===*/
 /*====================================================================*/
 
 /* A way to make limited debuginfo queries on a per-mapped-object
@@ -215,7 +263,7 @@ typedef  struct _DebugInfo  DebugInfo;
 
 /* Returns NULL if the DebugInfo isn't found.  It doesn't matter if
    debug info is present or not. */
-DebugInfo* VG_(find_DebugInfo) ( Addr a );
+DebugInfo* VG_(find_DebugInfo) ( DiEpoch ep, Addr a );
 
 /* Fish bits out of DebugInfos. */
 Addr          VG_(DebugInfo_get_text_avma)   ( const DebugInfo *di );
@@ -261,10 +309,10 @@ typedef
 const HChar* VG_(pp_SectKind)( VgSectKind kind );
 
 /* Given an address 'a', make a guess of which section of which object
-   it comes from.  If name is non-NULL, then the object's name is put
-   into *name. The returned name is persistent as long as the debuginfo
-   it belongs to isn't discarded. */
-VgSectKind VG_(DebugInfo_sect_kind)( /*OUT*/const HChar** name, Addr a);
+   it comes from.  If objname is non-NULL, then the object's name is put
+   into *objname. This only looks in debug info, it does not examine
+   the address space manager mapped files. */
+VgSectKind VG_(DebugInfo_sect_kind)( /*OUT*/const HChar** objname, Addr a);
 
 
 #endif   // __PUB_TOOL_DEBUGINFO_H
