@@ -1396,9 +1396,15 @@ int main(void)
    GO(SYS_swapcontext, "2s 2m");
    SY(SYS_swapcontext, x0+1, x0+2); FAIL;
 
+#if (FREEBSD_VERS >= FREEBSD_13_1)
+   /* SYS_freebsd13_swapoff                 424 */
+   GO(SYS_freebsd13_swapoff, "1s 1m");
+   SY(SYS_freebsd13_swapoff, x0+1); FAIL;
+#else
    /* SYS_swapoff                 424 */
    GO(SYS_swapoff, "1s 1m");
    SY(SYS_swapoff, x0+1); FAIL;
+#endif
 
    /* SYS___acl_get_link          425 */
    GO(SYS___acl_get_link, "3s 2m");
